import logging
from typing import Optional

from neutrino_client.client import _NeutrinoClient
from neutrino_client.neutrino_task import NeutrinoTask
from neutrino_client.task_pb2 import TaskInput

neutrino: Optional[_NeutrinoClient] = None


def init(dsn: str,
         secure: bool = True,
         host:str = "application",
         timeout_in_seconds: Optional[float] = 1.0,
         reconnect_on_connection_lost: Optional[bool] = False,
         keep_alive_time_ms: int = 30000,
         keepalive_timeout_ms: int = 5000):
    _NeutrinoClient(dsn=dsn,host=host, secure=secure, timeout_in_seconds=timeout_in_seconds,
                    reconnect_on_connection_lost=reconnect_on_connection_lost, keep_alive_time_ms=keep_alive_time_ms,
                    keepalive_timeout_ms=keepalive_timeout_ms)


def publish_task_info(task: NeutrinoTask):
    """
    Constructs TaskInput() object from param
    Invokes client.send_message(task_input)
    All kinds of exceptions are suppressed so that client is not affected
    """
    try:
        task_input = TaskInput()
        task_input.task_id = task.task_id
        task_input.task_name = task.task_name
        task_input.threshold = task.threshold
        task_input.threshold_unit = task.threshold_unit
        task_input.status = task.status
        task_input.type = task.type
        for k, v in task.additional_data.items():
            task_input.additional_data[k] = v
        _NeutrinoClient.send_message(task_input)
    except Exception as e:
        logging.info(f"neutrino: error while sending task info {str(e)}")


def get_grpc_channel():
    return _NeutrinoClient.get_channel()



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class TaskInput(_message.Message):
    __slots__ = ("task_id", "task_name", "timestamp", "type", "status", "threshold", "threshold_unit", "additional_data", "host")
    class AdditionalDataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    TASK_NAME_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    THRESHOLD_FIELD_NUMBER: _ClassVar[int]
    THRESHOLD_UNIT_FIELD_NUMBER: _ClassVar[int]
    ADDITIONAL_DATA_FIELD_NUMBER: _ClassVar[int]
    HOST_FIELD_NUMBER: _ClassVar[int]
    task_id: str
    task_name: str
    timestamp: str
    type: str
    status: str
    threshold: int
    threshold_unit: str
    additional_data: _containers.ScalarMap[str, str]
    host: str
    def __init__(self, task_id: _Optional[str] = ..., task_name: _Optional[str] = ..., timestamp: _Optional[str] = ..., type: _Optional[str] = ..., status: _Optional[str] = ..., threshold: _Optional[int] = ..., threshold_unit: _Optional[str] = ..., additional_data: _Optional[_Mapping[str, str]] = ..., host: _Optional[str] = ...) -> None: ...

class TaskOutput(_message.Message):
    __slots__ = ("response",)
    class ResponseEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    RESPONSE_FIELD_NUMBER: _ClassVar[int]
    response: _containers.ScalarMap[str, str]
    def __init__(self, response: _Optional[_Mapping[str, str]] = ...) -> None: ...



from typing import Optional

from pydantic import BaseModel


class NeutrinoTask(BaseModel):
    task_name: str
    task_id: str
    status: str
    threshold: Optional[int] = 1
    threshold_unit: Optional[str] = 'minutes'
    additional_data: Optional[dict[str, str]] = {}
    timestamp: Optional[str] = ''
    type: Optional[str] = ''



import logging
from typing import Optional

import grpc
from neutrino_client.task_pb2_grpc import TaskStub
from neutrino_client.task_pb2 import TaskInput
import logging as log


class _NeutrinoClient:
    _grpc_channel: Optional[grpc.Channel] = None
    _secure = True
    _host = "application"
    _stub: Optional[TaskStub] = None
    _timeout_in_seconds: float = 1.0
    _reconnect_on_connection_lost: bool = False
    _dsn: str = None
    _keepalive_time_ms: int
    _keepalive_timeout_ms: int

    def __new__(cls, dsn: str,
                secure: bool,
                host:str,
                timeout_in_seconds: float,
                reconnect_on_connection_lost: bool,
                keep_alive_time_ms: int,
                keepalive_timeout_ms: int):
        """
        creates new gRPC channel and initializes the stub
        """
        cls._dsn = dsn
        cls._secure = secure
        cls._host = host
        cls._timeout_in_seconds = timeout_in_seconds
        cls._reconnect_on_connection_lost = reconnect_on_connection_lost
        cls._keepalive_time_ms = keep_alive_time_ms
        cls._keepalive_timeout_ms = keepalive_timeout_ms

        if cls._grpc_channel is None:
            try:
                cls._grpc_channel = cls._connect_to_neutrino_server(dsn)
            except Exception as e:
                log.info(f"neutrino:error creating grpc channel - {str(e)}")
                return

        if cls._stub is None:
            cls._stub = TaskStub(cls._grpc_channel)

        return cls._grpc_channel

    @classmethod
    def _connect_to_neutrino_server(cls, dsn: str) -> Optional[grpc.Channel]:
        try:
            options = [
                ('grpc.keepalive_time_ms', cls._keepalive_time_ms),
                ('grpc.keepalive_timeout_ms', cls._keepalive_timeout_ms),
            ]
            if cls._secure:
                logging.info("neutrino: connecting through secure channel")
                return grpc.secure_channel(dsn, grpc.ssl_channel_credentials(), options=options)
            logging.info("neutrino: connecting through insecure channel")
            return grpc.insecure_channel(dsn, options)
        except Exception as e:
            logging.info(f"error connect to grpc server {str(e)}")
            return

    @classmethod
    def _retry_init(cls):
        if cls._grpc_channel is None:
            cls._grpc_channel = cls._connect_to_neutrino_server(cls._dsn)

        if cls._stub is None:
            cls._stub = TaskStub(cls._grpc_channel)

    @classmethod
    def send_message(cls, task_input: TaskInput):
        """
        Invokes UpsertTaskInfo to send task information to gRPC server
        """
        if not cls._grpc_channel or not cls._stub:
            logging.info("neutrino: sdk not initialized, please initialize sdk")
            return

        try:
            task_input.host = cls._host
            cls._stub.UpsertTaskInfo(task_input, cls._timeout_in_seconds)
        except grpc.RpcError as e:
            logging.info(f"neutrino: error while publishing task info {str(e)}")
            if cls._reconnect_on_connection_lost:
                logging.info("neutrino: retrying to connect")
                cls._retry_init()
                cls._stub.UpsertTaskInfo(task_input, cls._timeout_in_seconds)
        except Exception as e:
            logging.error(f"neutrino: error while publishing task info {str(e)}")

    @classmethod
    def get_channel(cls):
        return cls._grpc_channel



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: task.proto
# Protobuf Python Version: 5.27.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    2,
    '',
    'task.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\ntask.proto\"\x89\x02\n\tTaskInput\x12\x0f\n\x07task_id\x18\x01 \x01(\t\x12\x11\n\ttask_name\x18\x02 \x01(\t\x12\x11\n\ttimestamp\x18\x03 \x01(\t\x12\x0c\n\x04type\x18\x04 \x01(\t\x12\x0e\n\x06status\x18\x05 \x01(\t\x12\x11\n\tthreshold\x18\x06 \x01(\x03\x12\x16\n\x0ethreshold_unit\x18\x07 \x01(\t\x12\x37\n\x0f\x61\x64\x64itional_data\x18\x08 \x03(\x0b\x32\x1e.TaskInput.AdditionalDataEntry\x12\x0c\n\x04host\x18\t \x01(\t\x1a\x35\n\x13\x41\x64\x64itionalDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"j\n\nTaskOutput\x12+\n\x08response\x18\x01 \x03(\x0b\x32\x19.TaskOutput.ResponseEntry\x1a/\n\rResponseEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x32\x31\n\x04Task\x12)\n\x0eUpsertTaskInfo\x12\n.TaskInput\x1a\x0b.TaskOutputB\x0fZ\r./translationb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'task_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\r./translation'
  _globals['_TASKINPUT_ADDITIONALDATAENTRY']._loaded_options = None
  _globals['_TASKINPUT_ADDITIONALDATAENTRY']._serialized_options = b'8\001'
  _globals['_TASKOUTPUT_RESPONSEENTRY']._loaded_options = None
  _globals['_TASKOUTPUT_RESPONSEENTRY']._serialized_options = b'8\001'
  _globals['_TASKINPUT']._serialized_start=15
  _globals['_TASKINPUT']._serialized_end=280
  _globals['_TASKINPUT_ADDITIONALDATAENTRY']._serialized_start=227
  _globals['_TASKINPUT_ADDITIONALDATAENTRY']._serialized_end=280
  _globals['_TASKOUTPUT']._serialized_start=282
  _globals['_TASKOUTPUT']._serialized_end=388
  _globals['_TASKOUTPUT_RESPONSEENTRY']._serialized_start=341
  _globals['_TASKOUTPUT_RESPONSEENTRY']._serialized_end=388
  _globals['_TASK']._serialized_start=390
  _globals['_TASK']._serialized_end=439
# @@protoc_insertion_point(module_scope)



import neutrino_client


def test_init():
    neutrino_client.init(dsn="localhost:9999")
    grpc_channel = neutrino_client.get_grpc_channel()
    assert grpc_channel


def test_capture_task():
    neutrino_client.init(dsn="localhost:9999")
    task = neutrino_client.NeutrinoTask(
        task_name="task name",
        task_id="abcd",
        status="IN_PROGRESS"
    )
    neutrino_client.publish_task_info(task)



from setuptools import find_packages, setup

setup(
    name='neutrino_client',
    packages=find_packages(include=['neutrino_client']),
    version='0.0.5',
    description='Neutrino client SDK',
    author='dpdzero',
    install_requires=['grpcio', 'grpcio-tools', 'protobuf', 'pydantic'],
    setup_requires=[],
    tests_require=['pytest'],
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.11',
        'Operating System :: OS Independent',
    ],
)

# python3 setup.py sdist bdist_wheel
